# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .track_add_params import TrackAddParams as TrackAddParams
from .track_list_params import TrackListParams as TrackListParams
from .track_add_response import TrackAddResponse as TrackAddResponse
from .image_list_response import ImageListResponse as ImageListResponse
from .image_update_params import ImageUpdateParams as ImageUpdateParams
from .track_remove_params import TrackRemoveParams as TrackRemoveParams
from .track_update_params import TrackUpdateParams as TrackUpdateParams
from .follower_check_params import FollowerCheckParams as FollowerCheckParams
from .track_remove_response import TrackRemoveResponse as TrackRemoveResponse
from .track_update_response import TrackUpdateResponse as TrackUpdateResponse
from .follower_follow_params import FollowerFollowParams as FollowerFollowParams
from .follower_check_response import FollowerCheckResponse as FollowerCheckResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.track_object import TrackObject

__all__ = ["RecommendationGetResponse", "Seed"]


class Seed(BaseModel):
    id: Optional[str] = None
    """The id used to select this seed.

    This will be the same as the string used in the `seed_artists`, `seed_tracks` or
    `seed_genres` parameter.
    """

    after_filtering_size: Optional[int] = FieldInfo(alias="afterFilteringSize", default=None)
    """
    The number of tracks available after min\\__\\** and max\\__\\** filters have been
    applied.
    """

    after_relinking_size: Optional[int] = FieldInfo(alias="afterRelinkingSize", default=None)
    """The number of tracks available after relinking for regional availability."""

    href: Optional[str] = None
    """A link to the full track or artist data for this seed.

    For tracks this will be a link to a Track Object. For artists a link to an
    Artist Object. For genre seeds, this value will be `null`.
    """

    initial_pool_size: Optional[int] = FieldInfo(alias="initialPoolSize", default=None)
    """The number of recommended tracks available for this seed."""

    type: Optional[str] = None
    """The entity type of this seed. One of `artist`, `track` or `genre`."""


class RecommendationGetResponse(BaseModel):
    seeds: List[Seed]
    """An array of recommendation seed objects."""

    tracks: List[TrackObject]
    """An array of track objects ordered according to the parameters supplied."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["CategoryGetPlaylistsParams"]


class CategoryGetPlaylistsParams(TypedDict, total=False):
    limit: int
    """The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50."""

    offset: int
    """The index of the first item to return.

    Default: 0 (the first item). Use with limit to get the next set of items.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .queue import (
    QueueResource,
    AsyncQueueResource,
    QueueResourceWithRawResponse,
    AsyncQueueResourceWithRawResponse,
    QueueResourceWithStreamingResponse,
    AsyncQueueResourceWithStreamingResponse,
)
from .player import (
    PlayerResource,
    AsyncPlayerResource,
    PlayerResourceWithRawResponse,
    AsyncPlayerResourceWithRawResponse,
    PlayerResourceWithStreamingResponse,
    AsyncPlayerResourceWithStreamingResponse,
)

__all__ = [
    "QueueResource",
    "AsyncQueueResource",
    "QueueResourceWithRawResponse",
    "AsyncQueueResourceWithRawResponse",
    "QueueResourceWithStreamingResponse",
    "AsyncQueueResourceWithStreamingResponse",
    "PlayerResource",
    "AsyncPlayerResource",
    "PlayerResourceWithRawResponse",
    "AsyncPlayerResourceWithRawResponse",
    "PlayerResourceWithStreamingResponse",
    "AsyncPlayerResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["EpisodeCheckParams"]


class EpisodeCheckParams(TypedDict, total=False):
    ids: Required[str]
    """
    A comma-separated list of the
    [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) for the
    episodes. Maximum: 50 IDs.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions

import httpx

from .._types import Body, Query, Headers, NotGiven, not_given
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.audio_analysis_retrieve_response import AudioAnalysisRetrieveResponse

__all__ = ["AudioAnalysisResource", "AsyncAudioAnalysisResource"]


class AudioAnalysisResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AudioAnalysisResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AudioAnalysisResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AudioAnalysisResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AudioAnalysisResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AudioAnalysisRetrieveResponse:
        """Get a low-level audio analysis for a track in the Spotify catalog.

        The audio
        analysis describes the track’s structure and musical content, including rhythm,
        pitch, and timbre.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
              track.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/audio-analysis/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AudioAnalysisRetrieveResponse,
        )


class AsyncAudioAnalysisResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAudioAnalysisResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncAudioAnalysisResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAudioAnalysisResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncAudioAnalysisResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AudioAnalysisRetrieveResponse:
        """Get a low-level audio analysis for a track in the Spotify catalog.

        The audio
        analysis describes the track’s structure and musical content, including rhythm,
        pitch, and timbre.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
              track.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/audio-analysis/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AudioAnalysisRetrieveResponse,
        )


class AudioAnalysisResourceWithRawResponse:
    def __init__(self, audio_analysis: AudioAnalysisResource) -> None:
        self._audio_analysis = audio_analysis

        self.retrieve = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                audio_analysis.retrieve,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncAudioAnalysisResourceWithRawResponse:
    def __init__(self, audio_analysis: AsyncAudioAnalysisResource) -> None:
        self._audio_analysis = audio_analysis

        self.retrieve = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                audio_analysis.retrieve,  # pyright: ignore[reportDeprecated],
            )
        )


class AudioAnalysisResourceWithStreamingResponse:
    def __init__(self, audio_analysis: AudioAnalysisResource) -> None:
        self._audio_analysis = audio_analysis

        self.retrieve = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                audio_analysis.retrieve,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncAudioAnalysisResourceWithStreamingResponse:
    def __init__(self, audio_analysis: AsyncAudioAnalysisResource) -> None:
        self._audio_analysis = audio_analysis

        self.retrieve = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                audio_analysis.retrieve,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PlayerToggleShuffleParams"]


class PlayerToggleShuffleParams(TypedDict, total=False):
    state: Required[bool]
    """
    **true** : Shuffle user's playback.<br/> **false** : Do not shuffle user's
    playback.
    """

    device_id: str
    """The id of the device this command is targeting.

    If not supplied, the user's currently active device is the target.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel
from .context_object import ContextObject
from ..shared.track_object import TrackObject

__all__ = ["PlayerListRecentlyPlayedResponse"]


class PlayerListRecentlyPlayedResponse(BaseModel):
    context: Optional[ContextObject] = None
    """The context the track was played from."""

    played_at: Optional[datetime] = None
    """The date and time the track was played."""

    track: Optional[TrackObject] = None
    """The track the user listened to."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["NarratorObject"]


class NarratorObject(BaseModel):
    name: Optional[str] = None
    """The name of the Narrator."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["MarketListResponse"]


class MarketListResponse(BaseModel):
    markets: Optional[List[str]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["FollowingCheckParams"]


class FollowingCheckParams(TypedDict, total=False):
    ids: Required[str]
    """
    A comma-separated list of the artist or the user
    [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) to check. For
    example: `ids=74ASZWbe4lXaubB36ztrGX,08td7MxkoHQkXnWAYD8d6Q`. A maximum of 50
    IDs can be sent in one request.
    """

    type: Required[Literal["artist", "user"]]
    """The ID type: either `artist` or `user`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ResumePointObject"]


class ResumePointObject(BaseModel):
    fully_played: Optional[bool] = None
    """Whether or not the episode has been fully played by the user."""

    resume_position_ms: Optional[int] = None
    """The user's most recent position in the episode in milliseconds."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .shared.image_object import ImageObject
from .shared.external_url_object import ExternalURLObject
from .shared.album_restriction_object import AlbumRestrictionObject
from .shared.simplified_artist_object import SimplifiedArtistObject

__all__ = ["ArtistListAlbumsResponse"]


class ArtistListAlbumsResponse(BaseModel):
    id: str
    """
    The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
    album.
    """

    album_group: Literal["album", "single", "compilation", "appears_on"]
    """This field describes the relationship between the artist and the album."""

    album_type: Literal["album", "single", "compilation"]
    """The type of the album."""

    artists: List[SimplifiedArtistObject]
    """The artists of the album.

    Each artist object includes a link in `href` to more detailed information about
    the artist.
    """

    available_markets: List[str]
    """
    The markets in which the album is available:
    [ISO 3166-1 alpha-2 country codes](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
    _**NOTE**: an album is considered available in a market when at least 1 of its
    tracks is available in that market._
    """

    external_urls: ExternalURLObject
    """Known external URLs for this album."""

    href: str
    """A link to the Web API endpoint providing full details of the album."""

    images: List[ImageObject]
    """The cover art for the album in various sizes, widest first."""

    name: str
    """The name of the album.

    In case of an album takedown, the value may be an empty string.
    """

    release_date: str
    """The date the album was first released."""

    release_date_precision: Literal["year", "month", "day"]
    """The precision with which `release_date` value is known."""

    total_tracks: int
    """The number of tracks in the album."""

    type: Literal["album"]
    """The object type."""

    uri: str
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for the
    album.
    """

    restrictions: Optional[AlbumRestrictionObject] = None
    """Included in the response when a content restriction is applied."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from spotted import Spotted, AsyncSpotted
from tests.utils import assert_matches_type
from spotted.types import (
    AlbumRetrieveResponse,
    AlbumBulkRetrieveResponse,
)
from spotted.pagination import SyncCursorURLPage, AsyncCursorURLPage
from spotted.types.shared import SimplifiedTrackObject

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAlbums:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Spotted) -> None:
        album = client.albums.retrieve(
            id="4aawyAB9vmqN3uQ7FjRGTy",
        )
        assert_matches_type(AlbumRetrieveResponse, album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Spotted) -> None:
        album = client.albums.retrieve(
            id="4aawyAB9vmqN3uQ7FjRGTy",
            market="ES",
        )
        assert_matches_type(AlbumRetrieveResponse, album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Spotted) -> None:
        response = client.albums.with_raw_response.retrieve(
            id="4aawyAB9vmqN3uQ7FjRGTy",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        album = response.parse()
        assert_matches_type(AlbumRetrieveResponse, album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Spotted) -> None:
        with client.albums.with_streaming_response.retrieve(
            id="4aawyAB9vmqN3uQ7FjRGTy",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            album = response.parse()
            assert_matches_type(AlbumRetrieveResponse, album, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Spotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.albums.with_raw_response.retrieve(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_bulk_retrieve(self, client: Spotted) -> None:
        album = client.albums.bulk_retrieve(
            ids="382ObEPsp2rxGrnsizN5TX,1A2GTWGtFfWp7KSQTwWOyo,2noRn2Aes5aoNVsU6iWThc",
        )
        assert_matches_type(AlbumBulkRetrieveResponse, album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_bulk_retrieve_with_all_params(self, client: Spotted) -> None:
        album = client.albums.bulk_retrieve(
            ids="382ObEPsp2rxGrnsizN5TX,1A2GTWGtFfWp7KSQTwWOyo,2noRn2Aes5aoNVsU6iWThc",
            market="ES",
        )
        assert_matches_type(AlbumBulkRetrieveResponse, album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_bulk_retrieve(self, client: Spotted) -> None:
        response = client.albums.with_raw_response.bulk_retrieve(
            ids="382ObEPsp2rxGrnsizN5TX,1A2GTWGtFfWp7KSQTwWOyo,2noRn2Aes5aoNVsU6iWThc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        album = response.parse()
        assert_matches_type(AlbumBulkRetrieveResponse, album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_bulk_retrieve(self, client: Spotted) -> None:
        with client.albums.with_streaming_response.bulk_retrieve(
            ids="382ObEPsp2rxGrnsizN5TX,1A2GTWGtFfWp7KSQTwWOyo,2noRn2Aes5aoNVsU6iWThc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            album = response.parse()
            assert_matches_type(AlbumBulkRetrieveResponse, album, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_tracks(self, client: Spotted) -> None:
        album = client.albums.list_tracks(
            id="4aawyAB9vmqN3uQ7FjRGTy",
        )
        assert_matches_type(SyncCursorURLPage[SimplifiedTrackObject], album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_tracks_with_all_params(self, client: Spotted) -> None:
        album = client.albums.list_tracks(
            id="4aawyAB9vmqN3uQ7FjRGTy",
            limit=10,
            market="ES",
            offset=5,
        )
        assert_matches_type(SyncCursorURLPage[SimplifiedTrackObject], album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_tracks(self, client: Spotted) -> None:
        response = client.albums.with_raw_response.list_tracks(
            id="4aawyAB9vmqN3uQ7FjRGTy",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        album = response.parse()
        assert_matches_type(SyncCursorURLPage[SimplifiedTrackObject], album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_tracks(self, client: Spotted) -> None:
        with client.albums.with_streaming_response.list_tracks(
            id="4aawyAB9vmqN3uQ7FjRGTy",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            album = response.parse()
            assert_matches_type(SyncCursorURLPage[SimplifiedTrackObject], album, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list_tracks(self, client: Spotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.albums.with_raw_response.list_tracks(
                id="",
            )


class TestAsyncAlbums:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSpotted) -> None:
        album = await async_client.albums.retrieve(
            id="4aawyAB9vmqN3uQ7FjRGTy",
        )
        assert_matches_type(AlbumRetrieveResponse, album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncSpotted) -> None:
        album = await async_client.albums.retrieve(
            id="4aawyAB9vmqN3uQ7FjRGTy",
            market="ES",
        )
        assert_matches_type(AlbumRetrieveResponse, album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSpotted) -> None:
        response = await async_client.albums.with_raw_response.retrieve(
            id="4aawyAB9vmqN3uQ7FjRGTy",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        album = await response.parse()
        assert_matches_type(AlbumRetrieveResponse, album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSpotted) -> None:
        async with async_client.albums.with_streaming_response.retrieve(
            id="4aawyAB9vmqN3uQ7FjRGTy",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            album = await response.parse()
            assert_matches_type(AlbumRetrieveResponse, album, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSpotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.albums.with_raw_response.retrieve(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_bulk_retrieve(self, async_client: AsyncSpotted) -> None:
        album = await async_client.albums.bulk_retrieve(
            ids="382ObEPsp2rxGrnsizN5TX,1A2GTWGtFfWp7KSQTwWOyo,2noRn2Aes5aoNVsU6iWThc",
        )
        assert_matches_type(AlbumBulkRetrieveResponse, album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_bulk_retrieve_with_all_params(self, async_client: AsyncSpotted) -> None:
        album = await async_client.albums.bulk_retrieve(
            ids="382ObEPsp2rxGrnsizN5TX,1A2GTWGtFfWp7KSQTwWOyo,2noRn2Aes5aoNVsU6iWThc",
            market="ES",
        )
        assert_matches_type(AlbumBulkRetrieveResponse, album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_bulk_retrieve(self, async_client: AsyncSpotted) -> None:
        response = await async_client.albums.with_raw_response.bulk_retrieve(
            ids="382ObEPsp2rxGrnsizN5TX,1A2GTWGtFfWp7KSQTwWOyo,2noRn2Aes5aoNVsU6iWThc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        album = await response.parse()
        assert_matches_type(AlbumBulkRetrieveResponse, album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_bulk_retrieve(self, async_client: AsyncSpotted) -> None:
        async with async_client.albums.with_streaming_response.bulk_retrieve(
            ids="382ObEPsp2rxGrnsizN5TX,1A2GTWGtFfWp7KSQTwWOyo,2noRn2Aes5aoNVsU6iWThc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            album = await response.parse()
            assert_matches_type(AlbumBulkRetrieveResponse, album, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_tracks(self, async_client: AsyncSpotted) -> None:
        album = await async_client.albums.list_tracks(
            id="4aawyAB9vmqN3uQ7FjRGTy",
        )
        assert_matches_type(AsyncCursorURLPage[SimplifiedTrackObject], album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_tracks_with_all_params(self, async_client: AsyncSpotted) -> None:
        album = await async_client.albums.list_tracks(
            id="4aawyAB9vmqN3uQ7FjRGTy",
            limit=10,
            market="ES",
            offset=5,
        )
        assert_matches_type(AsyncCursorURLPage[SimplifiedTrackObject], album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_tracks(self, async_client: AsyncSpotted) -> None:
        response = await async_client.albums.with_raw_response.list_tracks(
            id="4aawyAB9vmqN3uQ7FjRGTy",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        album = await response.parse()
        assert_matches_type(AsyncCursorURLPage[SimplifiedTrackObject], album, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_tracks(self, async_client: AsyncSpotted) -> None:
        async with async_client.albums.with_streaming_response.list_tracks(
            id="4aawyAB9vmqN3uQ7FjRGTy",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            album = await response.parse()
            assert_matches_type(AsyncCursorURLPage[SimplifiedTrackObject], album, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list_tracks(self, async_client: AsyncSpotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.albums.with_raw_response.list_tracks(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["PlayerGetStateParams"]


class PlayerGetStateParams(TypedDict, total=False):
    additional_types: str
    """
    A comma-separated list of item types that your client supports besides the
    default `track` type. Valid types are: `track` and `episode`.<br/> _**Note**:
    This parameter was introduced to allow existing clients to maintain their
    current behaviour and might be deprecated in the future._<br/> In addition to
    providing this parameter, make sure that your client properly handles cases of
    new types in the future by checking against the `type` field of each object.
    """

    market: str
    """
    An
    [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
    If a country code is specified, only content that is available in that market
    will be returned.<br/> If a valid user access token is specified in the request
    header, the country associated with the user account will take priority over
    this parameter.<br/> _**Note**: If neither market or user country are provided,
    the content is considered unavailable for the client._<br/> Users can view the
    country that is associated with their account in the
    [account settings](https://www.spotify.com/account/overview/).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from ..shared.external_url_object import ExternalURLObject

__all__ = ["ContextObject"]


class ContextObject(BaseModel):
    external_urls: Optional[ExternalURLObject] = None
    """External URLs for this context."""

    href: Optional[str] = None
    """A link to the Web API endpoint providing full details of the track."""

    type: Optional[str] = None
    """The object type, e.g. "artist", "playlist", "album", "show"."""

    uri: Optional[str] = None
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for the
    context.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .external_url_object import ExternalURLObject

__all__ = ["LinkedTrackObject"]


class LinkedTrackObject(BaseModel):
    id: Optional[str] = None
    """
    The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
    track.
    """

    external_urls: Optional[ExternalURLObject] = None
    """Known external URLs for this track."""

    href: Optional[str] = None
    """A link to the Web API endpoint providing full details of the track."""

    type: Optional[str] = None
    """The object type: "track"."""

    uri: Optional[str] = None
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for the
    track.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from spotted import Spotted, AsyncSpotted
from tests.utils import assert_matches_type
from spotted.types.me import (
    PlayerGetStateResponse,
    PlayerGetDevicesResponse,
    PlayerListRecentlyPlayedResponse,
    PlayerGetCurrentlyPlayingResponse,
)
from spotted.pagination import SyncCursorURLPage, AsyncCursorURLPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPlayer:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_currently_playing(self, client: Spotted) -> None:
        player = client.me.player.get_currently_playing()
        assert_matches_type(PlayerGetCurrentlyPlayingResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_currently_playing_with_all_params(self, client: Spotted) -> None:
        player = client.me.player.get_currently_playing(
            additional_types="additional_types",
            market="ES",
        )
        assert_matches_type(PlayerGetCurrentlyPlayingResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_currently_playing(self, client: Spotted) -> None:
        response = client.me.player.with_raw_response.get_currently_playing()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = response.parse()
        assert_matches_type(PlayerGetCurrentlyPlayingResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_currently_playing(self, client: Spotted) -> None:
        with client.me.player.with_streaming_response.get_currently_playing() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = response.parse()
            assert_matches_type(PlayerGetCurrentlyPlayingResponse, player, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_devices(self, client: Spotted) -> None:
        player = client.me.player.get_devices()
        assert_matches_type(PlayerGetDevicesResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_devices(self, client: Spotted) -> None:
        response = client.me.player.with_raw_response.get_devices()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = response.parse()
        assert_matches_type(PlayerGetDevicesResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_devices(self, client: Spotted) -> None:
        with client.me.player.with_streaming_response.get_devices() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = response.parse()
            assert_matches_type(PlayerGetDevicesResponse, player, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_state(self, client: Spotted) -> None:
        player = client.me.player.get_state()
        assert_matches_type(PlayerGetStateResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_state_with_all_params(self, client: Spotted) -> None:
        player = client.me.player.get_state(
            additional_types="additional_types",
            market="ES",
        )
        assert_matches_type(PlayerGetStateResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_state(self, client: Spotted) -> None:
        response = client.me.player.with_raw_response.get_state()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = response.parse()
        assert_matches_type(PlayerGetStateResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_state(self, client: Spotted) -> None:
        with client.me.player.with_streaming_response.get_state() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = response.parse()
            assert_matches_type(PlayerGetStateResponse, player, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_recently_played(self, client: Spotted) -> None:
        player = client.me.player.list_recently_played()
        assert_matches_type(SyncCursorURLPage[PlayerListRecentlyPlayedResponse], player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_recently_played_with_all_params(self, client: Spotted) -> None:
        player = client.me.player.list_recently_played(
            after=1484811043508,
            before=0,
            limit=10,
        )
        assert_matches_type(SyncCursorURLPage[PlayerListRecentlyPlayedResponse], player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_recently_played(self, client: Spotted) -> None:
        response = client.me.player.with_raw_response.list_recently_played()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = response.parse()
        assert_matches_type(SyncCursorURLPage[PlayerListRecentlyPlayedResponse], player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_recently_played(self, client: Spotted) -> None:
        with client.me.player.with_streaming_response.list_recently_played() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = response.parse()
            assert_matches_type(SyncCursorURLPage[PlayerListRecentlyPlayedResponse], player, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_pause_playback(self, client: Spotted) -> None:
        player = client.me.player.pause_playback()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_pause_playback_with_all_params(self, client: Spotted) -> None:
        player = client.me.player.pause_playback(
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_pause_playback(self, client: Spotted) -> None:
        response = client.me.player.with_raw_response.pause_playback()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_pause_playback(self, client: Spotted) -> None:
        with client.me.player.with_streaming_response.pause_playback() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_seek_to_position(self, client: Spotted) -> None:
        player = client.me.player.seek_to_position(
            position_ms=25000,
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_seek_to_position_with_all_params(self, client: Spotted) -> None:
        player = client.me.player.seek_to_position(
            position_ms=25000,
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_seek_to_position(self, client: Spotted) -> None:
        response = client.me.player.with_raw_response.seek_to_position(
            position_ms=25000,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_seek_to_position(self, client: Spotted) -> None:
        with client.me.player.with_streaming_response.seek_to_position(
            position_ms=25000,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_set_repeat_mode(self, client: Spotted) -> None:
        player = client.me.player.set_repeat_mode(
            state="context",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_set_repeat_mode_with_all_params(self, client: Spotted) -> None:
        player = client.me.player.set_repeat_mode(
            state="context",
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_set_repeat_mode(self, client: Spotted) -> None:
        response = client.me.player.with_raw_response.set_repeat_mode(
            state="context",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_set_repeat_mode(self, client: Spotted) -> None:
        with client.me.player.with_streaming_response.set_repeat_mode(
            state="context",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_set_volume(self, client: Spotted) -> None:
        player = client.me.player.set_volume(
            volume_percent=50,
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_set_volume_with_all_params(self, client: Spotted) -> None:
        player = client.me.player.set_volume(
            volume_percent=50,
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_set_volume(self, client: Spotted) -> None:
        response = client.me.player.with_raw_response.set_volume(
            volume_percent=50,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_set_volume(self, client: Spotted) -> None:
        with client.me.player.with_streaming_response.set_volume(
            volume_percent=50,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_skip_next(self, client: Spotted) -> None:
        player = client.me.player.skip_next()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_skip_next_with_all_params(self, client: Spotted) -> None:
        player = client.me.player.skip_next(
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_skip_next(self, client: Spotted) -> None:
        response = client.me.player.with_raw_response.skip_next()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_skip_next(self, client: Spotted) -> None:
        with client.me.player.with_streaming_response.skip_next() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_skip_previous(self, client: Spotted) -> None:
        player = client.me.player.skip_previous()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_skip_previous_with_all_params(self, client: Spotted) -> None:
        player = client.me.player.skip_previous(
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_skip_previous(self, client: Spotted) -> None:
        response = client.me.player.with_raw_response.skip_previous()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_skip_previous(self, client: Spotted) -> None:
        with client.me.player.with_streaming_response.skip_previous() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_start_playback(self, client: Spotted) -> None:
        player = client.me.player.start_playback()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_start_playback_with_all_params(self, client: Spotted) -> None:
        player = client.me.player.start_playback(
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
            context_uri="spotify:album:5ht7ItJgpBH7W6vJ5BqpPr",
            offset={"position": "bar"},
            position_ms=0,
            uris=["string"],
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_start_playback(self, client: Spotted) -> None:
        response = client.me.player.with_raw_response.start_playback()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_start_playback(self, client: Spotted) -> None:
        with client.me.player.with_streaming_response.start_playback() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_toggle_shuffle(self, client: Spotted) -> None:
        player = client.me.player.toggle_shuffle(
            state=True,
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_toggle_shuffle_with_all_params(self, client: Spotted) -> None:
        player = client.me.player.toggle_shuffle(
            state=True,
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_toggle_shuffle(self, client: Spotted) -> None:
        response = client.me.player.with_raw_response.toggle_shuffle(
            state=True,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_toggle_shuffle(self, client: Spotted) -> None:
        with client.me.player.with_streaming_response.toggle_shuffle(
            state=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_transfer(self, client: Spotted) -> None:
        player = client.me.player.transfer(
            device_ids=["74ASZWbe4lXaubB36ztrGX"],
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_transfer_with_all_params(self, client: Spotted) -> None:
        player = client.me.player.transfer(
            device_ids=["74ASZWbe4lXaubB36ztrGX"],
            play=True,
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_transfer(self, client: Spotted) -> None:
        response = client.me.player.with_raw_response.transfer(
            device_ids=["74ASZWbe4lXaubB36ztrGX"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_transfer(self, client: Spotted) -> None:
        with client.me.player.with_streaming_response.transfer(
            device_ids=["74ASZWbe4lXaubB36ztrGX"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True


class TestAsyncPlayer:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_currently_playing(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.get_currently_playing()
        assert_matches_type(PlayerGetCurrentlyPlayingResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_currently_playing_with_all_params(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.get_currently_playing(
            additional_types="additional_types",
            market="ES",
        )
        assert_matches_type(PlayerGetCurrentlyPlayingResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_currently_playing(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.player.with_raw_response.get_currently_playing()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = await response.parse()
        assert_matches_type(PlayerGetCurrentlyPlayingResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_currently_playing(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.player.with_streaming_response.get_currently_playing() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = await response.parse()
            assert_matches_type(PlayerGetCurrentlyPlayingResponse, player, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_devices(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.get_devices()
        assert_matches_type(PlayerGetDevicesResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_devices(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.player.with_raw_response.get_devices()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = await response.parse()
        assert_matches_type(PlayerGetDevicesResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_devices(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.player.with_streaming_response.get_devices() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = await response.parse()
            assert_matches_type(PlayerGetDevicesResponse, player, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_state(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.get_state()
        assert_matches_type(PlayerGetStateResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_state_with_all_params(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.get_state(
            additional_types="additional_types",
            market="ES",
        )
        assert_matches_type(PlayerGetStateResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_state(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.player.with_raw_response.get_state()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = await response.parse()
        assert_matches_type(PlayerGetStateResponse, player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_state(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.player.with_streaming_response.get_state() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = await response.parse()
            assert_matches_type(PlayerGetStateResponse, player, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_recently_played(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.list_recently_played()
        assert_matches_type(AsyncCursorURLPage[PlayerListRecentlyPlayedResponse], player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_recently_played_with_all_params(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.list_recently_played(
            after=1484811043508,
            before=0,
            limit=10,
        )
        assert_matches_type(AsyncCursorURLPage[PlayerListRecentlyPlayedResponse], player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_recently_played(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.player.with_raw_response.list_recently_played()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = await response.parse()
        assert_matches_type(AsyncCursorURLPage[PlayerListRecentlyPlayedResponse], player, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_recently_played(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.player.with_streaming_response.list_recently_played() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = await response.parse()
            assert_matches_type(AsyncCursorURLPage[PlayerListRecentlyPlayedResponse], player, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_pause_playback(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.pause_playback()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_pause_playback_with_all_params(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.pause_playback(
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_pause_playback(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.player.with_raw_response.pause_playback()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = await response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_pause_playback(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.player.with_streaming_response.pause_playback() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = await response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_seek_to_position(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.seek_to_position(
            position_ms=25000,
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_seek_to_position_with_all_params(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.seek_to_position(
            position_ms=25000,
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_seek_to_position(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.player.with_raw_response.seek_to_position(
            position_ms=25000,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = await response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_seek_to_position(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.player.with_streaming_response.seek_to_position(
            position_ms=25000,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = await response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_set_repeat_mode(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.set_repeat_mode(
            state="context",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_set_repeat_mode_with_all_params(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.set_repeat_mode(
            state="context",
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_set_repeat_mode(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.player.with_raw_response.set_repeat_mode(
            state="context",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = await response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_set_repeat_mode(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.player.with_streaming_response.set_repeat_mode(
            state="context",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = await response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_set_volume(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.set_volume(
            volume_percent=50,
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_set_volume_with_all_params(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.set_volume(
            volume_percent=50,
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_set_volume(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.player.with_raw_response.set_volume(
            volume_percent=50,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = await response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_set_volume(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.player.with_streaming_response.set_volume(
            volume_percent=50,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = await response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_skip_next(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.skip_next()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_skip_next_with_all_params(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.skip_next(
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_skip_next(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.player.with_raw_response.skip_next()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = await response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_skip_next(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.player.with_streaming_response.skip_next() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = await response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_skip_previous(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.skip_previous()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_skip_previous_with_all_params(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.skip_previous(
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_skip_previous(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.player.with_raw_response.skip_previous()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = await response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_skip_previous(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.player.with_streaming_response.skip_previous() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = await response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_start_playback(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.start_playback()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_start_playback_with_all_params(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.start_playback(
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
            context_uri="spotify:album:5ht7ItJgpBH7W6vJ5BqpPr",
            offset={"position": "bar"},
            position_ms=0,
            uris=["string"],
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_start_playback(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.player.with_raw_response.start_playback()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = await response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_start_playback(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.player.with_streaming_response.start_playback() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = await response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_toggle_shuffle(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.toggle_shuffle(
            state=True,
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_toggle_shuffle_with_all_params(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.toggle_shuffle(
            state=True,
            device_id="0d1841b0976bae2a3a310dd74c0f3df354899bc8",
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_toggle_shuffle(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.player.with_raw_response.toggle_shuffle(
            state=True,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = await response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_toggle_shuffle(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.player.with_streaming_response.toggle_shuffle(
            state=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = await response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_transfer(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.transfer(
            device_ids=["74ASZWbe4lXaubB36ztrGX"],
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_transfer_with_all_params(self, async_client: AsyncSpotted) -> None:
        player = await async_client.me.player.transfer(
            device_ids=["74ASZWbe4lXaubB36ztrGX"],
            play=True,
        )
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_transfer(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.player.with_raw_response.transfer(
            device_ids=["74ASZWbe4lXaubB36ztrGX"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        player = await response.parse()
        assert player is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_transfer(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.player.with_streaming_response.transfer(
            device_ids=["74ASZWbe4lXaubB36ztrGX"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            player = await response.parse()
            assert player is None

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .show_base import ShowBase
from .image_object import ImageObject
from .external_url_object import ExternalURLObject
from .resume_point_object import ResumePointObject
from .episode_restriction_object import EpisodeRestrictionObject

__all__ = ["EpisodeObject"]


class EpisodeObject(BaseModel):
    id: str
    """
    The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
    episode.
    """

    audio_preview_url: Optional[str] = None
    """A URL to a 30 second preview (MP3 format) of the episode.

    `null` if not available.
    """

    description: str
    """A description of the episode.

    HTML tags are stripped away from this field, use `html_description` field in
    case HTML tags are needed.
    """

    duration_ms: int
    """The episode length in milliseconds."""

    explicit: bool
    """
    Whether or not the episode has explicit content (true = yes it does; false = no
    it does not OR unknown).
    """

    external_urls: ExternalURLObject
    """External URLs for this episode."""

    href: str
    """A link to the Web API endpoint providing full details of the episode."""

    html_description: str
    """A description of the episode. This field may contain HTML tags."""

    images: List[ImageObject]
    """The cover art for the episode in various sizes, widest first."""

    is_externally_hosted: bool
    """True if the episode is hosted outside of Spotify's CDN."""

    is_playable: bool
    """True if the episode is playable in the given market. Otherwise false."""

    languages: List[str]
    """
    A list of the languages used in the episode, identified by their
    [ISO 639-1](https://en.wikipedia.org/wiki/ISO_639) code.
    """

    name: str
    """The name of the episode."""

    release_date: str
    """The date the episode was first released, for example `"1981-12-15"`.

    Depending on the precision, it might be shown as `"1981"` or `"1981-12"`.
    """

    release_date_precision: Literal["year", "month", "day"]
    """The precision with which `release_date` value is known."""

    show: ShowBase
    """The show on which the episode belongs."""

    type: Literal["episode"]
    """The object type."""

    uri: str
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for the
    episode.
    """

    language: Optional[str] = None
    """
    The language used in the episode, identified by a
    [ISO 639](https://en.wikipedia.org/wiki/ISO_639) code. This field is deprecated
    and might be removed in the future. Please use the `languages` field instead.
    """

    restrictions: Optional[EpisodeRestrictionObject] = None
    """Included in the response when a content restriction is applied."""

    resume_point: Optional[ResumePointObject] = None
    """The user's most recent position in the episode.

    Set if the supplied access token is a user token and has the scope
    'user-read-playback-position'.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["QueueAddParams"]


class QueueAddParams(TypedDict, total=False):
    uri: Required[str]
    """The uri of the item to add to the queue. Must be a track or an episode uri."""

    device_id: str
    """The id of the device this command is targeting.

    If not supplied, the user's currently active device is the target.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["PlaylistTracksRefObject"]


class PlaylistTracksRefObject(BaseModel):
    href: Optional[str] = None
    """
    A link to the Web API endpoint where full details of the playlist's tracks can
    be retrieved.
    """

    total: Optional[int] = None
    """Number of tracks in the playlist."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["ShowSaveParams"]


class ShowSaveParams(TypedDict, total=False):
    ids: SequenceNotStr[str]
    """
    A JSON array of the
    [Spotify IDs](https://developer.spotify.com/documentation/web-api/#spotify-uris-and-ids).
    A maximum of 50 items can be specified in one request. _Note: if the `ids`
    parameter is present in the query string, any IDs listed here in the body will
    be ignored._
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .me import (
    MeResource,
    AsyncMeResource,
    MeResourceWithRawResponse,
    AsyncMeResourceWithRawResponse,
    MeResourceWithStreamingResponse,
    AsyncMeResourceWithStreamingResponse,
)
from .top import (
    TopResource,
    AsyncTopResource,
    TopResourceWithRawResponse,
    AsyncTopResourceWithRawResponse,
    TopResourceWithStreamingResponse,
    AsyncTopResourceWithStreamingResponse,
)
from .shows import (
    ShowsResource,
    AsyncShowsResource,
    ShowsResourceWithRawResponse,
    AsyncShowsResourceWithRawResponse,
    ShowsResourceWithStreamingResponse,
    AsyncShowsResourceWithStreamingResponse,
)
from .albums import (
    AlbumsResource,
    AsyncAlbumsResource,
    AlbumsResourceWithRawResponse,
    AsyncAlbumsResourceWithRawResponse,
    AlbumsResourceWithStreamingResponse,
    AsyncAlbumsResourceWithStreamingResponse,
)
from .player import (
    PlayerResource,
    AsyncPlayerResource,
    PlayerResourceWithRawResponse,
    AsyncPlayerResourceWithRawResponse,
    PlayerResourceWithStreamingResponse,
    AsyncPlayerResourceWithStreamingResponse,
)
from .tracks import (
    TracksResource,
    AsyncTracksResource,
    TracksResourceWithRawResponse,
    AsyncTracksResourceWithRawResponse,
    TracksResourceWithStreamingResponse,
    AsyncTracksResourceWithStreamingResponse,
)
from .episodes import (
    EpisodesResource,
    AsyncEpisodesResource,
    EpisodesResourceWithRawResponse,
    AsyncEpisodesResourceWithRawResponse,
    EpisodesResourceWithStreamingResponse,
    AsyncEpisodesResourceWithStreamingResponse,
)
from .following import (
    FollowingResource,
    AsyncFollowingResource,
    FollowingResourceWithRawResponse,
    AsyncFollowingResourceWithRawResponse,
    FollowingResourceWithStreamingResponse,
    AsyncFollowingResourceWithStreamingResponse,
)
from .playlists import (
    PlaylistsResource,
    AsyncPlaylistsResource,
    PlaylistsResourceWithRawResponse,
    AsyncPlaylistsResourceWithRawResponse,
    PlaylistsResourceWithStreamingResponse,
    AsyncPlaylistsResourceWithStreamingResponse,
)
from .audiobooks import (
    AudiobooksResource,
    AsyncAudiobooksResource,
    AudiobooksResourceWithRawResponse,
    AsyncAudiobooksResourceWithRawResponse,
    AudiobooksResourceWithStreamingResponse,
    AsyncAudiobooksResourceWithStreamingResponse,
)

__all__ = [
    "AudiobooksResource",
    "AsyncAudiobooksResource",
    "AudiobooksResourceWithRawResponse",
    "AsyncAudiobooksResourceWithRawResponse",
    "AudiobooksResourceWithStreamingResponse",
    "AsyncAudiobooksResourceWithStreamingResponse",
    "PlaylistsResource",
    "AsyncPlaylistsResource",
    "PlaylistsResourceWithRawResponse",
    "AsyncPlaylistsResourceWithRawResponse",
    "PlaylistsResourceWithStreamingResponse",
    "AsyncPlaylistsResourceWithStreamingResponse",
    "TopResource",
    "AsyncTopResource",
    "TopResourceWithRawResponse",
    "AsyncTopResourceWithRawResponse",
    "TopResourceWithStreamingResponse",
    "AsyncTopResourceWithStreamingResponse",
    "AlbumsResource",
    "AsyncAlbumsResource",
    "AlbumsResourceWithRawResponse",
    "AsyncAlbumsResourceWithRawResponse",
    "AlbumsResourceWithStreamingResponse",
    "AsyncAlbumsResourceWithStreamingResponse",
    "TracksResource",
    "AsyncTracksResource",
    "TracksResourceWithRawResponse",
    "AsyncTracksResourceWithRawResponse",
    "TracksResourceWithStreamingResponse",
    "AsyncTracksResourceWithStreamingResponse",
    "EpisodesResource",
    "AsyncEpisodesResource",
    "EpisodesResourceWithRawResponse",
    "AsyncEpisodesResourceWithRawResponse",
    "EpisodesResourceWithStreamingResponse",
    "AsyncEpisodesResourceWithStreamingResponse",
    "ShowsResource",
    "AsyncShowsResource",
    "ShowsResourceWithRawResponse",
    "AsyncShowsResourceWithRawResponse",
    "ShowsResourceWithStreamingResponse",
    "AsyncShowsResourceWithStreamingResponse",
    "FollowingResource",
    "AsyncFollowingResource",
    "FollowingResourceWithRawResponse",
    "AsyncFollowingResourceWithRawResponse",
    "FollowingResourceWithStreamingResponse",
    "AsyncFollowingResourceWithStreamingResponse",
    "PlayerResource",
    "AsyncPlayerResource",
    "PlayerResourceWithRawResponse",
    "AsyncPlayerResourceWithRawResponse",
    "PlayerResourceWithStreamingResponse",
    "AsyncPlayerResourceWithStreamingResponse",
    "MeResource",
    "AsyncMeResource",
    "MeResourceWithRawResponse",
    "AsyncMeResourceWithRawResponse",
    "MeResourceWithStreamingResponse",
    "AsyncMeResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.me import audiobook_list_params, audiobook_save_params, audiobook_check_params, audiobook_remove_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorURLPage, AsyncCursorURLPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.me.audiobook_list_response import AudiobookListResponse
from ...types.me.audiobook_check_response import AudiobookCheckResponse

__all__ = ["AudiobooksResource", "AsyncAudiobooksResource"]


class AudiobooksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AudiobooksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AudiobooksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AudiobooksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AudiobooksResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorURLPage[AudiobookListResponse]:
        """
        Get a list of the audiobooks saved in the current Spotify user's 'Your Music'
        library.

        Args:
          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/me/audiobooks",
            page=SyncCursorURLPage[AudiobookListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    audiobook_list_params.AudiobookListParams,
                ),
            ),
            model=AudiobookListResponse,
        )

    def check(
        self,
        *,
        ids: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AudiobookCheckResponse:
        """
        Check if one or more audiobooks are already saved in the current Spotify user's
        library.

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
              `ids=18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ`. Maximum: 50 IDs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/me/audiobooks/contains",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"ids": ids}, audiobook_check_params.AudiobookCheckParams),
            ),
            cast_to=AudiobookCheckResponse,
        )

    def remove(
        self,
        *,
        ids: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove one or more audiobooks from the Spotify user's library.

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
              `ids=18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ`. Maximum: 50 IDs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            "/me/audiobooks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"ids": ids}, audiobook_remove_params.AudiobookRemoveParams),
            ),
            cast_to=NoneType,
        )

    def save(
        self,
        *,
        ids: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Save one or more audiobooks to the current Spotify user's library.

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
              `ids=18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ`. Maximum: 50 IDs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            "/me/audiobooks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"ids": ids}, audiobook_save_params.AudiobookSaveParams),
            ),
            cast_to=NoneType,
        )


class AsyncAudiobooksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAudiobooksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncAudiobooksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAudiobooksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncAudiobooksResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AudiobookListResponse, AsyncCursorURLPage[AudiobookListResponse]]:
        """
        Get a list of the audiobooks saved in the current Spotify user's 'Your Music'
        library.

        Args:
          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/me/audiobooks",
            page=AsyncCursorURLPage[AudiobookListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    audiobook_list_params.AudiobookListParams,
                ),
            ),
            model=AudiobookListResponse,
        )

    async def check(
        self,
        *,
        ids: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AudiobookCheckResponse:
        """
        Check if one or more audiobooks are already saved in the current Spotify user's
        library.

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
              `ids=18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ`. Maximum: 50 IDs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/me/audiobooks/contains",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"ids": ids}, audiobook_check_params.AudiobookCheckParams),
            ),
            cast_to=AudiobookCheckResponse,
        )

    async def remove(
        self,
        *,
        ids: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove one or more audiobooks from the Spotify user's library.

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
              `ids=18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ`. Maximum: 50 IDs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            "/me/audiobooks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"ids": ids}, audiobook_remove_params.AudiobookRemoveParams),
            ),
            cast_to=NoneType,
        )

    async def save(
        self,
        *,
        ids: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Save one or more audiobooks to the current Spotify user's library.

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
              `ids=18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ`. Maximum: 50 IDs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            "/me/audiobooks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"ids": ids}, audiobook_save_params.AudiobookSaveParams),
            ),
            cast_to=NoneType,
        )


class AudiobooksResourceWithRawResponse:
    def __init__(self, audiobooks: AudiobooksResource) -> None:
        self._audiobooks = audiobooks

        self.list = to_raw_response_wrapper(
            audiobooks.list,
        )
        self.check = to_raw_response_wrapper(
            audiobooks.check,
        )
        self.remove = to_raw_response_wrapper(
            audiobooks.remove,
        )
        self.save = to_raw_response_wrapper(
            audiobooks.save,
        )


class AsyncAudiobooksResourceWithRawResponse:
    def __init__(self, audiobooks: AsyncAudiobooksResource) -> None:
        self._audiobooks = audiobooks

        self.list = async_to_raw_response_wrapper(
            audiobooks.list,
        )
        self.check = async_to_raw_response_wrapper(
            audiobooks.check,
        )
        self.remove = async_to_raw_response_wrapper(
            audiobooks.remove,
        )
        self.save = async_to_raw_response_wrapper(
            audiobooks.save,
        )


class AudiobooksResourceWithStreamingResponse:
    def __init__(self, audiobooks: AudiobooksResource) -> None:
        self._audiobooks = audiobooks

        self.list = to_streamed_response_wrapper(
            audiobooks.list,
        )
        self.check = to_streamed_response_wrapper(
            audiobooks.check,
        )
        self.remove = to_streamed_response_wrapper(
            audiobooks.remove,
        )
        self.save = to_streamed_response_wrapper(
            audiobooks.save,
        )


class AsyncAudiobooksResourceWithStreamingResponse:
    def __init__(self, audiobooks: AsyncAudiobooksResource) -> None:
        self._audiobooks = audiobooks

        self.list = async_to_streamed_response_wrapper(
            audiobooks.list,
        )
        self.check = async_to_streamed_response_wrapper(
            audiobooks.check,
        )
        self.remove = async_to_streamed_response_wrapper(
            audiobooks.remove,
        )
        self.save = async_to_streamed_response_wrapper(
            audiobooks.save,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["DeviceObject"]


class DeviceObject(BaseModel):
    id: Optional[str] = None
    """The device ID.

    This ID is unique and persistent to some extent. However, this is not guaranteed
    and any cached `device_id` should periodically be cleared out and refetched as
    necessary.
    """

    is_active: Optional[bool] = None
    """If this device is the currently active device."""

    is_private_session: Optional[bool] = None
    """If this device is currently in a private session."""

    is_restricted: Optional[bool] = None
    """Whether controlling this device is restricted.

    At present if this is "true" then no Web API commands will be accepted by this
    device.
    """

    name: Optional[str] = None
    """A human-readable name for the device.

    Some devices have a name that the user can configure (e.g. \"Loudest speaker\")
    and some devices have a generic name associated with the manufacturer or device
    model.
    """

    supports_volume: Optional[bool] = None
    """If this device can be used to set the volume."""

    type: Optional[str] = None
    """Device type, such as "computer", "smartphone" or "speaker"."""

    volume_percent: Optional[int] = None
    """The current volume in percent."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import audiobook_retrieve_params, audiobook_bulk_retrieve_params, audiobook_list_chapters_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorURLPage, AsyncCursorURLPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.simplified_chapter_object import SimplifiedChapterObject
from ..types.audiobook_retrieve_response import AudiobookRetrieveResponse
from ..types.audiobook_bulk_retrieve_response import AudiobookBulkRetrieveResponse

__all__ = ["AudiobooksResource", "AsyncAudiobooksResource"]


class AudiobooksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AudiobooksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AudiobooksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AudiobooksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AudiobooksResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AudiobookRetrieveResponse:
        """Get Spotify catalog information for a single audiobook.

        Audiobooks are only
        available within the US, UK, Canada, Ireland, New Zealand and Australia markets.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
              audiobook.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/audiobooks/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"market": market}, audiobook_retrieve_params.AudiobookRetrieveParams),
            ),
            cast_to=AudiobookRetrieveResponse,
        )

    def bulk_retrieve(
        self,
        *,
        ids: str,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AudiobookBulkRetrieveResponse:
        """
        Get Spotify catalog information for several audiobooks identified by their
        Spotify IDs. Audiobooks are only available within the US, UK, Canada, Ireland,
        New Zealand and Australia markets.

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
              `ids=18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ`. Maximum: 50 IDs.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/audiobooks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "market": market,
                    },
                    audiobook_bulk_retrieve_params.AudiobookBulkRetrieveParams,
                ),
            ),
            cast_to=AudiobookBulkRetrieveResponse,
        )

    def list_chapters(
        self,
        id: str,
        *,
        limit: int | Omit = omit,
        market: str | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorURLPage[SimplifiedChapterObject]:
        """Get Spotify catalog information about an audiobook's chapters.

        Audiobooks are
        only available within the US, UK, Canada, Ireland, New Zealand and Australia
        markets.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
              audiobook.

          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/audiobooks/{id}/chapters",
            page=SyncCursorURLPage[SimplifiedChapterObject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "market": market,
                        "offset": offset,
                    },
                    audiobook_list_chapters_params.AudiobookListChaptersParams,
                ),
            ),
            model=SimplifiedChapterObject,
        )


class AsyncAudiobooksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAudiobooksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncAudiobooksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAudiobooksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncAudiobooksResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AudiobookRetrieveResponse:
        """Get Spotify catalog information for a single audiobook.

        Audiobooks are only
        available within the US, UK, Canada, Ireland, New Zealand and Australia markets.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
              audiobook.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/audiobooks/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"market": market}, audiobook_retrieve_params.AudiobookRetrieveParams
                ),
            ),
            cast_to=AudiobookRetrieveResponse,
        )

    async def bulk_retrieve(
        self,
        *,
        ids: str,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AudiobookBulkRetrieveResponse:
        """
        Get Spotify catalog information for several audiobooks identified by their
        Spotify IDs. Audiobooks are only available within the US, UK, Canada, Ireland,
        New Zealand and Australia markets.

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
              `ids=18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ`. Maximum: 50 IDs.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/audiobooks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "ids": ids,
                        "market": market,
                    },
                    audiobook_bulk_retrieve_params.AudiobookBulkRetrieveParams,
                ),
            ),
            cast_to=AudiobookBulkRetrieveResponse,
        )

    def list_chapters(
        self,
        id: str,
        *,
        limit: int | Omit = omit,
        market: str | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[SimplifiedChapterObject, AsyncCursorURLPage[SimplifiedChapterObject]]:
        """Get Spotify catalog information about an audiobook's chapters.

        Audiobooks are
        only available within the US, UK, Canada, Ireland, New Zealand and Australia
        markets.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
              audiobook.

          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/audiobooks/{id}/chapters",
            page=AsyncCursorURLPage[SimplifiedChapterObject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "market": market,
                        "offset": offset,
                    },
                    audiobook_list_chapters_params.AudiobookListChaptersParams,
                ),
            ),
            model=SimplifiedChapterObject,
        )


class AudiobooksResourceWithRawResponse:
    def __init__(self, audiobooks: AudiobooksResource) -> None:
        self._audiobooks = audiobooks

        self.retrieve = to_raw_response_wrapper(
            audiobooks.retrieve,
        )
        self.bulk_retrieve = to_raw_response_wrapper(
            audiobooks.bulk_retrieve,
        )
        self.list_chapters = to_raw_response_wrapper(
            audiobooks.list_chapters,
        )


class AsyncAudiobooksResourceWithRawResponse:
    def __init__(self, audiobooks: AsyncAudiobooksResource) -> None:
        self._audiobooks = audiobooks

        self.retrieve = async_to_raw_response_wrapper(
            audiobooks.retrieve,
        )
        self.bulk_retrieve = async_to_raw_response_wrapper(
            audiobooks.bulk_retrieve,
        )
        self.list_chapters = async_to_raw_response_wrapper(
            audiobooks.list_chapters,
        )


class AudiobooksResourceWithStreamingResponse:
    def __init__(self, audiobooks: AudiobooksResource) -> None:
        self._audiobooks = audiobooks

        self.retrieve = to_streamed_response_wrapper(
            audiobooks.retrieve,
        )
        self.bulk_retrieve = to_streamed_response_wrapper(
            audiobooks.bulk_retrieve,
        )
        self.list_chapters = to_streamed_response_wrapper(
            audiobooks.list_chapters,
        )


class AsyncAudiobooksResourceWithStreamingResponse:
    def __init__(self, audiobooks: AsyncAudiobooksResource) -> None:
        self._audiobooks = audiobooks

        self.retrieve = async_to_streamed_response_wrapper(
            audiobooks.retrieve,
        )
        self.bulk_retrieve = async_to_streamed_response_wrapper(
            audiobooks.bulk_retrieve,
        )
        self.list_chapters = async_to_streamed_response_wrapper(
            audiobooks.list_chapters,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from spotted import Spotted, AsyncSpotted
from tests.utils import assert_matches_type
from spotted.types.me import (
    AudiobookListResponse,
    AudiobookCheckResponse,
)
from spotted.pagination import SyncCursorURLPage, AsyncCursorURLPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAudiobooks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Spotted) -> None:
        audiobook = client.me.audiobooks.list()
        assert_matches_type(SyncCursorURLPage[AudiobookListResponse], audiobook, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Spotted) -> None:
        audiobook = client.me.audiobooks.list(
            limit=10,
            offset=5,
        )
        assert_matches_type(SyncCursorURLPage[AudiobookListResponse], audiobook, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Spotted) -> None:
        response = client.me.audiobooks.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audiobook = response.parse()
        assert_matches_type(SyncCursorURLPage[AudiobookListResponse], audiobook, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Spotted) -> None:
        with client.me.audiobooks.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            audiobook = response.parse()
            assert_matches_type(SyncCursorURLPage[AudiobookListResponse], audiobook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_check(self, client: Spotted) -> None:
        audiobook = client.me.audiobooks.check(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        )
        assert_matches_type(AudiobookCheckResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_check(self, client: Spotted) -> None:
        response = client.me.audiobooks.with_raw_response.check(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audiobook = response.parse()
        assert_matches_type(AudiobookCheckResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_check(self, client: Spotted) -> None:
        with client.me.audiobooks.with_streaming_response.check(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            audiobook = response.parse()
            assert_matches_type(AudiobookCheckResponse, audiobook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_remove(self, client: Spotted) -> None:
        audiobook = client.me.audiobooks.remove(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        )
        assert audiobook is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_remove(self, client: Spotted) -> None:
        response = client.me.audiobooks.with_raw_response.remove(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audiobook = response.parse()
        assert audiobook is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_remove(self, client: Spotted) -> None:
        with client.me.audiobooks.with_streaming_response.remove(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            audiobook = response.parse()
            assert audiobook is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_save(self, client: Spotted) -> None:
        audiobook = client.me.audiobooks.save(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        )
        assert audiobook is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_save(self, client: Spotted) -> None:
        response = client.me.audiobooks.with_raw_response.save(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audiobook = response.parse()
        assert audiobook is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_save(self, client: Spotted) -> None:
        with client.me.audiobooks.with_streaming_response.save(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            audiobook = response.parse()
            assert audiobook is None

        assert cast(Any, response.is_closed) is True


class TestAsyncAudiobooks:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncSpotted) -> None:
        audiobook = await async_client.me.audiobooks.list()
        assert_matches_type(AsyncCursorURLPage[AudiobookListResponse], audiobook, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSpotted) -> None:
        audiobook = await async_client.me.audiobooks.list(
            limit=10,
            offset=5,
        )
        assert_matches_type(AsyncCursorURLPage[AudiobookListResponse], audiobook, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.audiobooks.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audiobook = await response.parse()
        assert_matches_type(AsyncCursorURLPage[AudiobookListResponse], audiobook, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.audiobooks.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            audiobook = await response.parse()
            assert_matches_type(AsyncCursorURLPage[AudiobookListResponse], audiobook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_check(self, async_client: AsyncSpotted) -> None:
        audiobook = await async_client.me.audiobooks.check(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        )
        assert_matches_type(AudiobookCheckResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_check(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.audiobooks.with_raw_response.check(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audiobook = await response.parse()
        assert_matches_type(AudiobookCheckResponse, audiobook, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_check(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.audiobooks.with_streaming_response.check(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            audiobook = await response.parse()
            assert_matches_type(AudiobookCheckResponse, audiobook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_remove(self, async_client: AsyncSpotted) -> None:
        audiobook = await async_client.me.audiobooks.remove(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        )
        assert audiobook is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_remove(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.audiobooks.with_raw_response.remove(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audiobook = await response.parse()
        assert audiobook is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_remove(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.audiobooks.with_streaming_response.remove(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            audiobook = await response.parse()
            assert audiobook is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_save(self, async_client: AsyncSpotted) -> None:
        audiobook = await async_client.me.audiobooks.save(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        )
        assert audiobook is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_save(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.audiobooks.with_raw_response.save(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        audiobook = await response.parse()
        assert audiobook is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_save(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.audiobooks.with_streaming_response.save(
            ids="18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ,7iHfbu1YPACw6oZPAFJtqe",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            audiobook = await response.parse()
            assert audiobook is None

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .category_list_params import CategoryListParams as CategoryListParams
from .category_list_response import CategoryListResponse as CategoryListResponse
from .category_retrieve_params import CategoryRetrieveParams as CategoryRetrieveParams
from .category_retrieve_response import CategoryRetrieveResponse as CategoryRetrieveResponse
from .category_get_playlists_params import CategoryGetPlaylistsParams as CategoryGetPlaylistsParams
from .category_get_playlists_response import CategoryGetPlaylistsResponse as CategoryGetPlaylistsResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from ..shared.image_object import ImageObject

__all__ = ["CategoryListResponse", "Categories", "CategoriesItem"]


class CategoriesItem(BaseModel):
    id: str
    """
    The [Spotify category ID](/documentation/web-api/concepts/spotify-uris-ids) of
    the category.
    """

    href: str
    """A link to the Web API endpoint returning full details of the category."""

    icons: List[ImageObject]
    """The category icon, in various sizes."""

    name: str
    """The name of the category."""


class Categories(BaseModel):
    href: str
    """A link to the Web API endpoint returning the full result of the request"""

    limit: int
    """
    The maximum number of items in the response (as set in the query or by default).
    """

    next: Optional[str] = None
    """URL to the next page of items. ( `null` if none)"""

    offset: int
    """The offset of the items returned (as set in the query or by default)"""

    previous: Optional[str] = None
    """URL to the previous page of items. ( `null` if none)"""

    total: int
    """The total number of items available to return."""

    items: Optional[List[CategoriesItem]] = None


class CategoryListResponse(BaseModel):
    categories: Categories



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .image_object import ImageObject
from .external_id_object import ExternalIDObject
from .external_url_object import ExternalURLObject
from .linked_track_object import LinkedTrackObject
from .album_restriction_object import AlbumRestrictionObject
from .simplified_artist_object import SimplifiedArtistObject
from .track_restriction_object import TrackRestrictionObject

__all__ = ["TrackObject", "Album"]


class Album(BaseModel):
    id: str
    """
    The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
    album.
    """

    album_type: Literal["album", "single", "compilation"]
    """The type of the album."""

    artists: List[SimplifiedArtistObject]
    """The artists of the album.

    Each artist object includes a link in `href` to more detailed information about
    the artist.
    """

    available_markets: List[str]
    """
    The markets in which the album is available:
    [ISO 3166-1 alpha-2 country codes](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
    _**NOTE**: an album is considered available in a market when at least 1 of its
    tracks is available in that market._
    """

    external_urls: ExternalURLObject
    """Known external URLs for this album."""

    href: str
    """A link to the Web API endpoint providing full details of the album."""

    images: List[ImageObject]
    """The cover art for the album in various sizes, widest first."""

    name: str
    """The name of the album.

    In case of an album takedown, the value may be an empty string.
    """

    release_date: str
    """The date the album was first released."""

    release_date_precision: Literal["year", "month", "day"]
    """The precision with which `release_date` value is known."""

    total_tracks: int
    """The number of tracks in the album."""

    type: Literal["album"]
    """The object type."""

    uri: str
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for the
    album.
    """

    restrictions: Optional[AlbumRestrictionObject] = None
    """Included in the response when a content restriction is applied."""


class TrackObject(BaseModel):
    id: Optional[str] = None
    """
    The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
    track.
    """

    album: Optional[Album] = None
    """The album on which the track appears.

    The album object includes a link in `href` to full information about the album.
    """

    artists: Optional[List[SimplifiedArtistObject]] = None
    """The artists who performed the track.

    Each artist object includes a link in `href` to more detailed information about
    the artist.
    """

    available_markets: Optional[List[str]] = None
    """
    A list of the countries in which the track can be played, identified by their
    [ISO 3166-1 alpha-2](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2) code.
    """

    disc_number: Optional[int] = None
    """The disc number (usually `1` unless the album consists of more than one disc)."""

    duration_ms: Optional[int] = None
    """The track length in milliseconds."""

    explicit: Optional[bool] = None
    """
    Whether or not the track has explicit lyrics ( `true` = yes it does; `false` =
    no it does not OR unknown).
    """

    external_ids: Optional[ExternalIDObject] = None
    """Known external IDs for the track."""

    external_urls: Optional[ExternalURLObject] = None
    """Known external URLs for this track."""

    href: Optional[str] = None
    """A link to the Web API endpoint providing full details of the track."""

    is_local: Optional[bool] = None
    """Whether or not the track is from a local file."""

    is_playable: Optional[bool] = None
    """
    Part of the response when
    [Track Relinking](/documentation/web-api/concepts/track-relinking) is applied.
    If `true`, the track is playable in the given market. Otherwise `false`.
    """

    linked_from: Optional[LinkedTrackObject] = None
    """
    Part of the response when
    [Track Relinking](/documentation/web-api/concepts/track-relinking) is applied,
    and the requested track has been replaced with different track. The track in the
    `linked_from` object contains information about the originally requested track.
    """

    name: Optional[str] = None
    """The name of the track."""

    popularity: Optional[int] = None
    """The popularity of the track.

    The value will be between 0 and 100, with 100 being the most popular.<br/>The
    popularity of a track is a value between 0 and 100, with 100 being the most
    popular. The popularity is calculated by algorithm and is based, in the most
    part, on the total number of plays the track has had and how recent those plays
    are.<br/>Generally speaking, songs that are being played a lot now will have a
    higher popularity than songs that were played a lot in the past. Duplicate
    tracks (e.g. the same track from a single and an album) are rated independently.
    Artist and album popularity is derived mathematically from track popularity.
    _**Note**: the popularity value may lag actual popularity by a few days: the
    value is not updated in real time._
    """

    preview_url: Optional[str] = None
    """A link to a 30 second preview (MP3 format) of the track. Can be `null`"""

    restrictions: Optional[TrackRestrictionObject] = None
    """Included in the response when a content restriction is applied."""

    track_number: Optional[int] = None
    """The number of the track.

    If an album has several discs, the track number is the number on the specified
    disc.
    """

    type: Optional[Literal["track"]] = None
    """The object type: "track"."""

    uri: Optional[str] = None
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for the
    track.
    """
